@@ -1,36 +1,96 @@
-/* Sniffit -- coded by Brecht Claerhout                                   */
-/*            Time wasted: too much                                       */
-/* formatted code and added support for hostname resolution - Godmar Back */ 
+/* Sniffit main program file                                              */
+/*   - by           : Brecht Claerhout                                    */
+/*   - getaddrbyname: Godmar Back                                         */
 
-#include "config.h"                                  /* Config header file */
+#include "sn_config.h"                                  /* Config header file */
+
+#include <unistd.h>
+#include <signal.h>
+#include <stdio.h>
+#include <netdb.h>
+#include <errno.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/time.h>
+#include <netinet/in.h>
+#ifdef INCLUDE_INTERFACE
+#include <sys/ipc.h>
+#include <sys/shm.h>
+#endif                 
+#include "pcap.h"  
+
+/************************ Own includes *************************************/
 #include "sn_defines.h"                                 /* Various defines */
+#include "sn_structs.h"                                  /* All structures */
+
 #include "sn_global.h"                                 /* some global defs */
-#include "sn_oldether.h"                 /* Leftover from 0.0.X and 0.1.X  */
-#include "sn_packets.h"                           /* My own packet structs */
 #include "sn_data.h"                                          /* data defs */
+
+#include "sn_packets.h"                           /* My own packet structs */
 #include "sn_logfile.h"
+#include "sn_cfgfile.h"                            /* Config file handling */
+
+#include "sn_plugins.h"                            /* Sniffit Plugins file */
+
 #ifdef INCLUDE_INTERFACE
 #include "sn_interface.h"                            /* all ncurses stuff  */
 #endif
-#include "sn_cfgfile.h"                            /* Config file handling */
+#ifdef GENERATION
+#include "sn_generation.h"                    /* Sniffit Packet generation */
+#endif
+
+#include "sniffit.h"                           /* definition of functions  */
 
 static char Copyright[]= 
-	"Sniffit - Brecht Claerhout - Copyright 1996";
+	"Sniffit - Brecht Claerhout - Copyright 1996-97";
 
 void quit (char *prog_name)                    /* Learn to use the program */
 {
 	printf(
-"usage: %s [-xdabvn] [-P proto] [-A char] [-p port]\n"
-"       [-l sniflen] [-L loglevel] [-F snifdevice]\n"
+"usage: %s [-xdabvnN] [-P proto] [-A char] [-p port]\n"
+"       [-l sniflen] [-L loglevel] [-F snifdevice] [-M plugin]\n"
 #ifdef INCLUDE_INTERFACE
 "       [-D tty]" 
 " (-t<Target IP> | -s<Source IP>)"
-" | -i | -c<config file>\n",
+" | -i | -c<config file>]\n",
 #else
-"       (-t<Target IP> | -s<Source IP>) | -c<config file>\n",
+"       (-t<Target IP> | -s<Source IP>) | -c<config file>]\n",
 #endif
-		prog_name); 
-	exit(0); 
+		prog_name);
+printf("Plugins Available:\n"); 
+#ifdef PLUGIN0_NAME
+printf("    0 -- %s\n",PLUGIN0_NAME);
+#endif
+#ifdef PLUGIN1_NAME
+printf("    1 -- %s\n",PLUGIN1_NAME);
+#endif
+#ifdef PLUGIN2_NAME
+printf("    2 -- %s\n",PLUGIN2_NAME);
+#endif
+#ifdef PLUGIN3_NAME
+printf("    3 -- %s\n",PLUGIN3_NAME);
+#endif
+#ifdef PLUGIN4_NAME
+printf("    4 -- %s\n",PLUGIN4_NAME);
+#endif
+#ifdef PLUGIN5_NAME
+printf("    5 -- %s\n",PLUGIN5_NAME);
+#endif
+#ifdef PLUGIN6_NAME
+printf("    6 -- %s\n",PLUGIN6_NAME);
+#endif
+#ifdef PLUGIN7_NAME
+printf("    7 -- %s\n",PLUGIN7_NAME);
+#endif
+#ifdef PLUGIN8_NAME
+printf("    8 -- %s\n",PLUGIN8_NAME);
+#endif
+#ifdef PLUGIN9_NAME
+printf("    9 -- %s\n",PLUGIN9_NAME);
+#endif
+
+exit(0); 
 }
 
 void my_exit (void)	{	exit(0);};
@@ -58,6 +118,47 @@ char *strlower (char *string)
 	return string;
 }
 
+void start_plugin(int PL_nr, struct Plugin_data *PL_d)
+{
+switch(PL_nr)
+	{
+#ifdef PLUGIN0_NAME
+	case 0: PLUGIN0(PL_d); break;
+#endif	
+#ifdef PLUGIN1_NAME
+	case 1: PLUGIN1(PL_d); break;
+#endif	
+#ifdef PLUGIN2_NAME
+	case 2: PLUGIN2(PL_d); break;
+#endif	
+#ifdef PLUGIN3_NAME
+	case 3: PLUGIN3(PL_d); break;
+#endif	
+#ifdef PLUGIN4_NAME
+	case 4: PLUGIN4(PL_d); break;
+#endif	
+#ifdef PLUGIN5_NAME
+	case 5: PLUGIN5(PL_d); break;
+#endif	
+#ifdef PLUGIN5_NAME
+	case 5: PLUGIN5(PL_d); break;
+#endif	
+#ifdef PLUGIN6_NAME
+	case 6: PLUGIN6(PL_d); break;
+#endif	
+#ifdef PLUGIN7_NAME
+	case 7: PLUGIN7(PL_d); break;
+#endif	
+#ifdef PLUGIN8_NAME
+	case 8: PLUGIN8(PL_d); break;
+#endif	
+#ifdef PLUGIN9_NAME
+	case 9: PLUGIN9(PL_d); break;
+#endif	
+	default: fprintf(stderr,"Plugin does not exist...\n"); exit(1); break;
+	}
+}
+
 void reset_all (void) 
 {
 	start_dynam=NULL;
@@ -145,6 +246,7 @@ void delete_dynam (char *file, char ptype,char do_file)
 		if(do_file!=0)	fclose(search_pointer->f);
 		start_dynam=search_pointer->next;
 		dynam_len--;
+		free(search_pointer);
 		return;
 		}
  	search_pointer=start_dynam;
@@ -257,12 +359,12 @@ for(i=0;i<len;i++)
 void print_iphead (struct IP_header *iphead, char icmp_or_plain)
 {
         int dummy;
-	u_char *so,*dest;
+	unsigned char *so,*dest;
 	
 	if(icmp_or_plain!=0)
 		printf("ICMP message concerned following IP packet:\n");
-	so=(u_char *)&(iphead->source);
-       	dest=(u_char *)&(iphead->destination);
+	so=(unsigned char *)&(iphead->source);
+       	dest=(unsigned char *)&(iphead->destination);
 	printf("from %u.%u.%u.%u to %u.%u.%u.%u\n",
 					so[0],so[1],so[2],so[3],
                                         dest[0],dest[1],dest[2],dest[3]);
@@ -275,7 +377,7 @@ void print_iphead (struct IP_header *iphead, char icmp_or_plain)
         dummy=ntohs(iphead->flag_offset); dummy>>=13;
         printf("FLAGS: %s %s     Time to live (secs): %d\n",
 	                                (dummy&IP_DF)?"DF":"--",
-                                        (dummy&IP_DELAY)?"MF":"--",
+                                        (dummy&IP_MF)?"MF":"--",
 	                                 iphead->TTL);
         if(iphead->protocol < 34)
                 printf("Protocol (%d): %s\n",iphead->protocol,
@@ -284,9 +386,9 @@ void print_iphead (struct IP_header *iphead, char icmp_or_plain)
 	printf("\n");
 }
 
-int check_packet(u_long ipaddr,
+int check_packet(unsigned long ipaddr,
 		const struct packetheader *p_header, 
-		const u_char *sp,
+		const unsigned char *sp,
 		char *file,
 		char *file2,
 		struct unwrap *info,
@@ -295,7 +397,7 @@ int check_packet(u_long ipaddr,
                                           /* MODE 0: -t   MODE 1: -s */
                                           /* MODE 2: -b              */ 
 {
-        u_char *so,*dest;
+        unsigned char *so,*dest;
 	char wc_so[20], wc_dest[20];
 	struct IP_header iphead;
 	struct TCP_header tcphead;
@@ -308,12 +410,17 @@ int check_packet(u_long ipaddr,
 
 	proto=unwrap_packet(sp, info); 
 	if(proto == NO_IP)	return DONT_EXAMINE; /* no use in trying */
+	if(proto == NO_IP_4)	return DONT_EXAMINE; /* no use in trying */
+
         memcpy(&iphead,(sp+PROTO_HEAD),sizeof(struct IP_header));
-	so=(u_char *)&(iphead.source);
-       	dest=(u_char *)&(iphead.destination);
+	so=(unsigned char *)&(iphead.source);
+       	dest=(unsigned char *)&(iphead.destination);
 
 	if((proto==TCP)&&(PROTOCOLS&F_TCP)) 
 		{
+#ifdef DEBUG_ONSCREEN
+printf("TCP Packet\n");
+#endif
 		memcpy(&tcphead,(sp+PROTO_HEAD+info->IP_len),
 						sizeof(struct TCP_header));
 		memcpy(detail,&tcphead,sizeof(struct TCP_header));
@@ -332,6 +439,9 @@ int check_packet(u_long ipaddr,
 		  {
 		  if (WILDCARD!=0) /* Wildcards */
 		    {   
+#ifdef DEBUG_ONSCREEN
+printf("WILDCARD\n");
+#endif
 		    sprintf(wc_so,"%u.%u.%u.%u",so[0],so[1],so[2],so[3]);
 		    sprintf(wc_dest,"%u.%u.%u.%u",dest[0],dest[1],dest[2],dest[3]);
 		    n_s = strlen(IP);
@@ -481,6 +591,10 @@ printf("Selected: %d\n",selected);
 		if( DEST_PORT && ntohs(tcphead.destination) != DEST_PORT) 
 			return DONT_EXAMINE; 	/* Check dest. PORT */
 
+#ifdef DEBUG_ONSCREEN
+printf("Packet accepted\n");
+#endif
+
                           /* inet_ntoa gave wrong output sometimes */
 		sprintf(file,"%u.%u.%u.%u.%u-%u.%u.%u.%u.%u",
 					so[0],so[1],so[2],so[3],
@@ -497,6 +611,11 @@ printf("Selected: %d\n",selected);
 						/* check for reset conn. */
 			return TCP_FINISH;            /* packet is a FIN */
 
+/* NEW TEST  */
+		if ((ntohs(tcphead.offset_flag) & RST) != 0)  
+						/* check for reset conn. */
+			return TCP_FINISH;            /* packet is a FIN */
+
 		/*
 		Used to be for speed, '-x' needs all info, so this too!
 		if (info->DATA_len == 0)
@@ -517,39 +636,213 @@ printf("Selected: %d\n",selected);
 		};
 	if((proto==UDP)&&(PROTOCOLS&F_UDP))       /* UDP packet checking */
 		{
+#ifdef DEBUG_ONSCREEN
+printf("UDP Packet\n");
+#endif
 		memcpy(&udphead,(sp+PROTO_HEAD+info->IP_len),
 						sizeof(struct UDP_header));
 		memcpy(detail,&udphead,sizeof(struct UDP_header));
+
+		if((WILDCARD==0)&&(CFG_FILE==0))     /* Selection criteria */
+		  {
+		  if (MODE == DEST && ipaddr != iphead.destination   /* -t */
+			||
+	   	      MODE == SOURCE && ipaddr != iphead.source      /* -s */
+			||
+		      MODE == BOTH && ipaddr != iphead.destination   /* -b */
+			      && ipaddr != iphead.source
+	   	      )  return DONT_EXAMINE; /* Check destination/source IP */
+		  }
+		else
+		  {
+		  if (WILDCARD!=0) /* Wildcards */
+		    {   
+		    sprintf(wc_so,"%u.%u.%u.%u",so[0],so[1],so[2],so[3]);
+		    sprintf(wc_dest,"%u.%u.%u.%u",dest[0],dest[1],dest[2],dest[3]);
+		    n_s = strlen(IP);
+		    if (MODE == DEST && (strncmp(wc_dest,IP,n_s)!=0) /* -t */
+	           		||
+	   	        MODE == SOURCE && (strncmp(wc_so,IP,n_s)!=0) /* -s */
+		        	||
+		        MODE == BOTH && (strncmp(wc_dest,IP,n_s)!=0) /* -b */
+			         && (strncmp(wc_so,IP,n_s)!=0)
+	   	        )   return DONT_EXAMINE; 
+					      /* Check destination/source IP */
+		    }
+		  else 	{ /* We are using the config file (4*Check) */
+		       	prior=0; selected=0;
+		        sprintf(wc_so,"%u.%u.%u.%u",so[0],so[1],so[2],so[3]);
+		        sprintf(wc_dest,"%u.%u.%u.%u",dest[0],dest[1],dest[2],dest[3]);
+#ifdef DEBUG_ONSCREEN
+printf("CONCERNING: %s  %d - %s  %d\n",wc_so,ntohs(udphead.source),wc_dest,ntohs(udphead.destination));
+#endif
+                        /* Highest prior last (backward+break for speed)  */
+			for(i=(select_from_length-1);i>=0;i--)
+			  {
+			  if(prior > select_from_list[i].priority) break;
+			  str_IP = select_from_list[i].host;
+#ifdef DEBUG_ONSCREEN
+printf("SF: %s  %d   Prior:%d\n",str_IP,select_from_list[i].port,select_from_list[i].priority);
+#endif
+		          if((select_from_list[i].port==0) || 
+			     (select_from_list[i].port==ntohs(udphead.source)))
+				{
+				if(select_from_list[i].wildcard==0)
+				  {                          /* NO wildcard */ 
+     				  if(strcmp(wc_so,str_IP)==0)
+					{selected=1; 
+		                        prior=select_from_list[i].priority;
+			                break;} 
+				  }
+				else
+				  {                            /* wildcard */
+     				  if(strncmp(wc_so,str_IP,strlen(str_IP))==0)
+				 	{selected=1; 
+		              		prior=select_from_list[i].priority;
+			      		break;} 
+				  }
+			        if(str_IP[0]==0)
+					{selected=1; break;};
+				}
+			  }
+
+			for(i=(select_to_length-1);i>=0;i--)
+			  {
+			  if(prior > select_to_list[i].priority) break;
+			  str_IP = select_to_list[i].host;
+#ifdef DEBUG_ONSCREEN
+printf("ST: %s  %d   Prior:%d\n",str_IP,select_to_list[i].port,select_to_list[i].priority);
+#endif
+		          if((select_to_list[i].port==0) || 
+			     (select_to_list[i].port==ntohs(udphead.destination)))
+				{
+				if(select_to_list[i].wildcard==0)
+				  {                          /* NO wildcard */ 
+     				  if(strcmp(wc_dest,str_IP)==0)
+					{selected=1;
+		                        prior=select_to_list[i].priority;
+			                break;} 
+				  }
+				else
+				  {                            /* wildcard */
+     				  if(strncmp(wc_dest,str_IP,strlen(str_IP))==0)
+				 	{selected=1; 
+		              		prior=select_to_list[i].priority;
+			      		break;} 
+				  }
+			        if(str_IP[0]==0)
+					{selected=1; break;};
+			        }
+			  }
+
+			for(i=(deselect_from_length-1);i>=0;i--)
+			  {
+			  if(prior > deselect_from_list[i].priority) break;
+			  str_IP = deselect_from_list[i].host;
+#ifdef DEBUG_ONSCREEN
+printf("DF: %s  %d   Prior:%d\n",str_IP,deselect_from_list[i].port,deselect_from_list[i].priority);
+#endif
+		          if((deselect_from_list[i].port==0) || 
+			     (deselect_from_list[i].port==ntohs(udphead.source)))
+				{
+				if(deselect_from_list[i].wildcard==0)
+				  {                          /* NO wildcard */ 
+     				  if(strcmp(wc_so,str_IP)==0)
+					{selected=0; 
+		                        prior=deselect_from_list[i].priority;
+			                break;} 
+				  }
+				else
+				  {                            /* wildcard */
+     				  if(strcmp(wc_so,str_IP)==0)
+				 	{selected=0; 
+		              		prior=deselect_from_list[i].priority;
+			      		break;} 
+				  }
+			        if(str_IP[0]==0)
+					{selected=0; break;};
+
+			        }
+			  }
+
+			for(i=(deselect_to_length-1);i>=0;i--)
+			  {
+			  if(prior > deselect_to_list[i].priority) break;
+			  str_IP = deselect_to_list[i].host;
+#ifdef DEBUG_ONSCREEN
+printf("DT: %s  %d   Prior:%d\n",str_IP,deselect_to_list[i].port,deselect_to_list[i].priority);
+#endif
+		          if((deselect_to_list[i].port==0) || 
+			     (deselect_to_list[i].port==ntohs(udphead.destination)))
+				{
+				if(deselect_to_list[i].wildcard==0)
+				  {                          /* NO wildcard */ 
+     				  if(strncmp(wc_dest,str_IP,strlen(str_IP))==0)
+					{selected=0; 
+		                        prior=deselect_to_list[i].priority;
+			                break;} 
+				  }
+				else
+				  {                            /* wildcard */
+     				  if(strncmp(wc_dest,str_IP,strlen(str_IP))==0)
+				 	{selected=0; 
+		              		prior=deselect_to_list[i].priority;
+			      		break;} 
+				  }
+			        if(str_IP[0]==0)
+					{selected=0; break;};
+
+			        }
+			  }
+
+#ifdef DEBUG_ONSCREEN
+printf("Selected: %d\n",selected);
+#endif
+                       	if(selected==0) return DONT_EXAMINE; 
+
+			}
+		  }
+
+		if( DEST_PORT && ntohs(udphead.destination) != DEST_PORT) 
+			return DONT_EXAMINE;	/* Check dest. PORT */
+
+                          /* inet_ntoa gave wrong output sometimes */
 		sprintf(file,"%u.%u.%u.%u.%u-%u.%u.%u.%u.%u",
 					so[0],so[1],so[2],so[3],
-					ntohs(udphead.source),
+					(unsigned short)ntohs(udphead.source),
 					dest[0],dest[1],dest[2],dest[3],
-					ntohs(udphead.destination));
-		return UDP_EXAMINE;
+					(unsigned short)ntohs(udphead.destination));
+		sprintf(file2,"%u.%u.%u.%u.%u-%u.%u.%u.%u.%u",
+					dest[0],dest[1],dest[2],dest[3],
+					(unsigned short)ntohs(udphead.destination),
+					so[0],so[1],so[2],so[3],
+					(unsigned short)ntohs(udphead.source));
+		return UDP_EXAMINE;                 /* interprete packet */
 		}
 	return DONT_EXAMINE;
 }
 
 /* Default Processing of packets */
-pcap_handler packethandler(	u_char *ipaddrpoint, 
+pcap_handler packethandler(unsigned char *ipaddrpoint, 
 			const struct packetheader *p_header, 
-			const u_char *sp) 
+			const unsigned char *sp) 
 { 
 	char filename[50], filename2[50],header[SNAPLEN];
 	FILE *f;
 	struct file_info *dummy_pointer;
-	u_char status=0;
+	unsigned char status=0;
 	char *help;
 	int dummy,finish;                    /* look out it's signed */
-	u_long datalen, position, total_length, i, n, ipaddr;
+	unsigned long datalen, position, total_length, i, n, ipaddr;
 	struct unwrap info;
         struct IP_header iphead;
 	struct TCP_header tcphead;
 	struct ICMP_header icmphead;
 	struct IP_header iphead_icmp;
         struct UDP_header udphead;
-
-	memcpy(&ipaddr,ipaddrpoint,sizeof(u_long));
+	struct Plugin_data PL_d, PL_dex;
+	
+	memcpy(&ipaddr,ipaddrpoint,sizeof(unsigned long));
    	
 	finish=check_packet(ipaddr,p_header,sp,filename,filename2,&info,header,SNIFMODE); 
         if(PROTOCOLS & F_IP)
@@ -558,6 +851,33 @@ pcap_handler packethandler(	u_char *ipaddrpoint,
         if(finish==DONT_EXAMINE) 
 		return;                         /* Packet is not for us */
 
+#ifdef DEBUG_ONSCREEN
+printf("Processing Packet (finish: %d)\n",finish);
+#endif
+
+	if((finish<10)||(finish>=20&&finish<30))         /* Start plugin */
+		{
+		/* fixing data structure */
+		memcpy(&(PL_d.PL_info),&info,sizeof(struct unwrap));
+                memcpy(&(PL_d.PL_iphead),(sp+PROTO_HEAD),sizeof(struct IP_header));
+		memcpy(&(PL_d.PL_tcphead),(sp+PROTO_HEAD+info.IP_len),sizeof(struct TCP_header));
+		memcpy(&(PL_d.PL_udphead),(sp+PROTO_HEAD+info.IP_len),sizeof(struct UDP_header));
+		if(PL_d.PL_iphead.protocol == TCP)		
+		  memcpy(PL_d.PL_data,(sp+PROTO_HEAD+info.IP_len+info.TCP_len),info.DATA_len);
+		if(PL_d.PL_iphead.protocol == UDP)		
+		  memcpy(PL_d.PL_data,(sp+PROTO_HEAD+info.IP_len+info.UDP_len),info.DATA_len);
+		memcpy(PL_d.PL_packet,(sp+PROTO_HEAD),
+			 info.IP_len+info.TCP_len+info.UDP_len+info.DATA_len);
+
+		/* starting all plugins */
+		for(i=0;i<10;i++)
+			if(Plugin_Active[i]==1)
+				{
+				memcpy(&PL_dex,&PL_d,sizeof(struct Plugin_data));
+				start_plugin(i,&PL_dex);
+				}
+		}
+
 	if((DUMPMODE & 32)&&(finish<10))	/* extended info TCP only */
 		{
 		memcpy(&tcphead,header,sizeof(struct TCP_header));
@@ -581,7 +901,7 @@ pcap_handler packethandler(	u_char *ipaddrpoint,
 		/* use return instead of else {if}, for later extention */
 		{
 		memcpy(&tcphead,header,sizeof(struct TCP_header));
-		switch(DUMPMODE & 223) 
+		switch(DUMPMODE & 223) /* without '32' bit 5 */ 
 		  {
 		  case 0:                     /* LOG mode */
 		  status=0;
@@ -600,7 +920,7 @@ pcap_handler packethandler(	u_char *ipaddrpoint,
 		  f=dummy_pointer->f;
 
 		  if(dummy_pointer->bytes<=SNIFLEN) {
-			const u_char *data = 
+			const unsigned char *data = 
 					sp+PROTO_HEAD+info.IP_len+info.TCP_len;
 			if(SNIFLEN!=0)
 				dummy_pointer->bytes+=info.DATA_len;       
@@ -626,7 +946,7 @@ pcap_handler packethandler(	u_char *ipaddrpoint,
 			total_length=info.IP_len+info.TCP_len+info.DATA_len;
 			n = 0;
 			for(i=0;i<total_length;i++) {
-				u_char c = sp[PROTO_HEAD+i];
+				unsigned char c = sp[PROTO_HEAD+i];
 				if(n > 75)
 					n=0, printf("\n");
 				if(DUMPMODE & 1)
@@ -636,9 +956,10 @@ pcap_handler packethandler(	u_char *ipaddrpoint,
 			}
 			printf("\n\n");
 			break;
-		
+	
+		  case 128: break; /* NO LOGGING MODE */
   		  default: 
-			printf("\nImpossible error! Sniffer Hartattack!\n");
+			printf("\nYou mixed incompatible options!\n");
 			exit(0); 
 		  }
 	      	return;
@@ -681,7 +1002,7 @@ pcap_handler packethandler(	u_char *ipaddrpoint,
 		  }
 		if(LOGLEVEL<30)          /* more sophisticated */
 		  {
-		  const u_char *data = sp+PROTO_HEAD+info.IP_len+info.TCP_len;
+		  const unsigned char *data = sp+PROTO_HEAD+info.IP_len+info.TCP_len;
 
 		  if(search_dynam(filename2, TCP)==NULL)
          	     {
@@ -862,7 +1183,7 @@ pcap_handler packethandler(	u_char *ipaddrpoint,
 			n = 0;
 			for(i=0;i<total_length;i++) 
 				{
-				u_char c = sp[PROTO_HEAD+i];
+				unsigned char c = sp[PROTO_HEAD+i];
 				if(n > 75)
 					n=0, printf("\n");
 				if(DUMPMODE & 1)
@@ -872,6 +1193,7 @@ pcap_handler packethandler(	u_char *ipaddrpoint,
 				}
 			printf("\n\n");
 			break;
+	  	  case 128: break; /* NO LOGGING MODE */
   		  default: 
 			printf("\nImpossible error! Sniffer Hartattack!\n");
 			exit(0);
@@ -884,17 +1206,20 @@ pcap_handler packethandler(	u_char *ipaddrpoint,
 #ifdef INCLUDE_INTERFACE                         /* Interactive packethandling */ 
 int check_mask (const struct packetheader *p_header, 
 		const unsigned char *sp,
-		char *conn_name,
+		char *conn_name, char *conn_name2,
 		struct unwrap *info)
                                           /* return -1 : packet not for us */
                                           /* else finish value             */
 {
-        u_char *so,*dest;
+	char helpstr1[20], helpstr2[20];
+        unsigned char *so,*dest;
 	struct IP_header iphead;
 	struct TCP_header tcphead;
 	int proto;
 
 	proto=unwrap_packet(sp, info);
+	if(proto == NO_IP)	return DONT_EXAMINE; /* no use in trying */
+	if(proto == NO_IP_4)	return DONT_EXAMINE; /* no use in trying */
 
 	(*IP_nr_of_packets)++;
 	if(proto==ICMP)  
@@ -921,16 +1246,27 @@ int check_mask (const struct packetheader *p_header,
 		return DONT_EXAMINE; 
 
                           /* inet_ntoa gave wrong output sometimes */
-	so=(u_char *)&(iphead.source);
-        dest=(u_char *)&(iphead.destination);
-	sprintf(conn_name,"from %u.%u.%u.%u %u to %u.%u.%u.%u %u",
-				so[0],so[1],so[2],so[3],
+	so=(unsigned char *)&(iphead.source);
+        dest=(unsigned char *)&(iphead.destination);
+
+	sprintf(helpstr1,"%u.%u.%u.%u",so[0],so[1],so[2],so[3]);
+	sprintf(helpstr2,"%u.%u.%u.%u",dest[0],dest[1],dest[2],dest[3]);
+
+	sprintf(conn_name,"%16s %5u  -> %16s %5u",
+				helpstr1,
 				ntohs(tcphead.source),
-				dest[0],dest[1],dest[2],dest[3],
+				helpstr2,
 				ntohs(tcphead.destination));
+	sprintf(conn_name2,"%16s %5u  -> %16s %5u",
+				helpstr2,
+				ntohs(tcphead.destination),
+				helpstr1,
+				ntohs(tcphead.source));
 
 	if ((ntohs(tcphead.offset_flag) & FIN) != 0)/* check for reset conn. */
 		return TCP_FINISH;                        /* packet is a FIN */
+	if ((ntohs(tcphead.offset_flag) & RST) != 0)/* check for reset conn. */
+		return TCP_FINISH;                        /* packet is a RST */
 	if (info->DATA_len == 0)
   		return DONT_EXAMINE;                   /* packet not for us */
 	return TCP_EXAMINE;                            /* interprete packet */
@@ -940,18 +1276,18 @@ pcap_handler interactive_packethandler(	char *dummy,
 			 	const struct packetheader *p_header, 
 				const unsigned char *sp) 
 { 
-	char conn_name[CONN_NAMELEN];
+	char conn_name[CONN_NAMELEN], conn_name2[CONN_NAMELEN];
 	int finish;                    /* look out it's signed */
 	struct unwrap info;
 
-	finish=check_mask(p_header,sp,conn_name,&info); 
+	finish=check_mask(p_header,sp,conn_name,conn_name2,&info); 
 	if(finish==DONT_EXAMINE) return;         /* Packet is not for us */
 
 	if(finish!=TCP_FINISH) /* finish: already logged, or to short to add */
 		add_itemlist(running_connections,conn_name);
 	if(strcmp(log_conn->log_enter, conn_name)==0)
 		{
-                const u_char *data=sp+PROTO_HEAD+info.IP_len+info.TCP_len;
+                const unsigned char *data=sp+PROTO_HEAD+info.IP_len+info.TCP_len;
  		if(*DATAlength+info.DATA_len < LENGTH_OF_INTERPROC_DATA)
 			{
 			memcpy((connection_data+*DATAlength),data,info.DATA_len);
@@ -959,12 +1295,15 @@ pcap_handler interactive_packethandler(	char *dummy,
 			}             
 		}
 	if(finish==TCP_FINISH)
+		{
 		del_itemlist(running_connections,conn_name);
+		del_itemlist(running_connections,conn_name2);
+		}
 	kill(getppid(),SIGUSR1);
 }
 #endif
 
-static u_long getaddrbyname(char *name)
+unsigned long getaddrbyname(char *name)
 {
     struct hostent *he;
 
@@ -984,9 +1323,9 @@ static u_long getaddrbyname(char *name)
 int main(int argc,char *argv[])
 {
 	char *dev, forced_dev[20], buffer[SNAPLEN];
-	int c;
-	u_long ipaddr, memsize;
-	int flag=0, doboth=0, FORCE_DEV=0;
+	int c,i;
+	unsigned long ipaddr, memsize;
+	int flag=0, doboth=0, FORCE_DEV=0, SUPPORTED=0;
 	extern char *optarg;
 
 	signal (SIGINT, my_exit);       /* controled CTRL-C exit */
@@ -995,7 +1334,9 @@ int main(int argc,char *argv[])
 	SNIFMODE=DUMPMODE=PROTOCOLS=ASC=WILDCARD=CFG_FILE=NO_CHKSUM=0;
 	LOGLEVEL=0;
 	Logfile[0]=0;
-	IP=logging_device=NULL;  
+	logging_device=NULL;  
+	IP=NULL;
+        for(i=0;i<10;i++)	Plugin_Active[i]=0; /* Active plugins */
 
 #ifdef DEBUG
 	if((debug_dev=fopen(DEBUG_DEVICE,"a"))<0)
@@ -1008,18 +1349,18 @@ int main(int argc,char *argv[])
 #endif
 
 #ifdef INCLUDE_INTERFACE        
-	while((c=getopt(argc,argv,"D:A:P:idp:l:xabt:s:F:c:nvL:"))!=-1) { 
+	while((c=getopt(argc,argv,"D:A:P:idp:l:xabt:s:F:c:nvL:M:N"))!=-1) { 
 #else
-	while((c=getopt(argc,argv,"A:P:dp:l:xabt:s:F:c:nvL:"))!=-1) { 
+	while((c=getopt(argc,argv,"A:P:dp:l:xabt:s:F:c:nvL:M:N"))!=-1) { 
 #endif
                                                     /* Argument treating */
   		switch(c) {
 			case 'v':
-				printf("Sniffit Version %s - Copyright 1996 Brecht Claerhout\n",VERSION);
+				printf("Sniffit Version %s - Copyright 1996-97 Brecht Claerhout\n",VERSION); 
 #ifdef __DATE__
 				printf("Binary build %s %s\n",__DATE__,__TIME__);
 #endif
-				exit(0);
+				quit(argv[0]);
 				break;
 			case 'd':
 				DUMPMODE|=1;
@@ -1030,6 +1371,9 @@ int main(int argc,char *argv[])
 			case 'x':
 				DUMPMODE|=32;
 				break;
+			case 'N':
+				DUMPMODE|=128;
+				break;
 			case 'p':
 				DEST_PORT=atoi(optarg);
 				break;
@@ -1073,6 +1417,10 @@ int main(int argc,char *argv[])
                         case 'n':
 				NO_CHKSUM=1;
                                 break;
+			case 'M':
+				if( (atoi(optarg)>=0) && (atoi(optarg)<=9) )
+				      Plugin_Active[atoi(optarg)]=1;
+				break;
 			case 'F':
 				strcpy(forced_dev,optarg);
                                 FORCE_DEV=1;
@@ -1089,8 +1437,15 @@ int main(int argc,char *argv[])
       			default : break;
 		}
 	}
+
+#ifdef ALLOW_SUID
+	if ((getuid()!=0)&&(geteuid()!=0))
+		printf("You should be root to run this program!\n"), exit(1);
+#else
 	if (getuid()!=0)
 		printf("You should be root to run this program!\n"), exit(1);
+#endif
+
 	if(flag!=1) 	quit(argv[0]);
 	if( (LOGLEVEL!=0) && (CFG_FILE==0) )	quit(argv[0]);
 
@@ -1099,12 +1454,12 @@ int main(int argc,char *argv[])
 	if(doboth) SNIFMODE=BOTH;
         if((SNIFMODE!=INTERACTIVE)&&(CFG_FILE==0))  
 		{
-/*		if(index(IP,'x')) For SunOS  */
-		if(strchr(IP,'x'))
+/*		if(index(IP,'@')) For SunOS  */
+		if(strchr(IP,'@'))
 		  {printf("Wildcard detected, IP nr. not checked...\n");
 		  WILDCARD=1;
-/*		  strcpy(index(IP,'x'),"\0"); */
-		  strcpy(strchr(IP,'x'),"\0");
+/*		  strcpy(index(IP,'@'),"\0"); */
+		  strcpy(strchr(IP,'@'),"\0");
 		  }
 		else
 		  {
@@ -1129,14 +1484,29 @@ int main(int argc,char *argv[])
 		PROTO_HEAD=FORCED_HEAD_LENGTH;
 	      	}
 
-	if(strstr(dev,ETH_DEV))                /* For expansion */
-		{PROTO_HEAD=ETHERHEAD;
-		printf("Supported ethernet device found. (%s)\n",dev); 
-		}
+	for(i=0;i<ETH_DEV_NR;i++)
+		{
+		if(strstr(dev,ETH_DEV[i]))                /* For expansion */
+			{PROTO_HEAD=ETHERHEAD;
+			printf("Supported ethernet device found. (%s)\n",dev); 
+			SUPPORTED=1;
+			break;}
+		};
 
-	if(strstr(dev,PPP_DEV))               
-		{PROTO_HEAD=PPPHEAD;
-		printf("Supported PPP device found. (%s)\n",dev); 
+	for(i=0;i<PPP_DEV_NR;i++)
+		{
+	        if(strstr(dev,PPP_DEV[i]))               
+			{PROTO_HEAD=PPPHEAD;
+			printf("Supported PPP device found. (%s)\n",dev); 
+			SUPPORTED=1;
+			break;}
+		};
+
+	if((SUPPORTED==0)&&(FORCE_DEV==0)) /* not supported & not forced */
+		{
+		printf("Network device found... BUT not known by Sniffit, use '-F <netw device>' option!\n");
+		printf("Read the README.FIRST on how to force network devices.\n");
+		exit(1);
 		}
 
 	if((dev_desc=pcap_open_live(dev,SNAPLEN,1,MSDELAY,NULL))==NULL)
@@ -1212,7 +1582,7 @@ int main(int argc,char *argv[])
 		else 
 		  printf("Sniffit.%s is up and running.... (Config File Used)\n\n",VERSION);
 		while(1)
-  		  if(pcap_dispatch(dev_desc,PACKETS,packethandler,(u_char *)&ipaddr)<0)
+  		  if(pcap_dispatch(dev_desc,PACKETS,packethandler,(unsigned char *)&ipaddr)<0)
     			printf("Capturing Packets Failed\n"), exit(0);
 #ifdef INCLUDE_INTERFACE
 		}
